#ifndef MSP_GL_SL_DEBUG_H_
#define MSP_GL_SL_DEBUG_H_

#include "syntax.h"
#include "visitor.h"

namespace Msp {
namespace GL {
namespace SL {

/** Creates a textual representation of the syntax tree.  The result is encoded
as UTF-8. */
class DumpTree: private TraversingVisitor
{
private:
	enum TreeChars
	{
		EMPTY = 0x20,
		STRAIGHT = 0x2502,     // │
		BRANCH = 0x251C,       // ├
		BRANCH_LAST = 0x2514,  // └
		REACH = 0x2574         // ╴
	};

	struct Branch
	{
		std::string text;
		Node *node;

		Branch(const char *t, Node *n = 0): text(t), node(n) { }
		Branch(const std::string &t, Node *n = 0): text(t), node(n) { }
		Branch(Node *n): node(n) { }
	};

	std::map<const Node *, unsigned> node_labels;
	std::string formatted;
	std::vector<TreeChars> tree;

public:
	std::string apply(Stage &);
	std::string apply(Node &n) { n.visit(*this); return formatted; }

private:
	void append(const std::string &);
	void append(const Node &, const std::string &);
	void append_subtree(const std::vector<Branch> &);
	void append_subtree(Node &);
	void begin_sub();
	void last_branch();
	void end_sub();
	unsigned get_label(const Node &);
	std::string format_type(TypeDeclaration *);

	template<typename T>
	typename T::const_iterator increment(typename T::const_iterator &, const T &);

	virtual void visit(Block &);
	virtual void visit(Literal &);
	virtual void visit(VariableReference &);
	virtual void visit(InterfaceBlockReference &);
	virtual void visit(MemberAccess &);
	virtual void visit(Swizzle &);
	virtual void visit(UnaryExpression &);
	virtual void visit(BinaryExpression &);
	virtual void visit(Assignment &);
	virtual void visit(TernaryExpression &);
	virtual void visit(FunctionCall &);
	virtual void visit(ExpressionStatement &);
	virtual void visit(Import &);
	virtual void visit(Precision &);
	virtual void visit(Layout &);
	virtual void visit(InterfaceLayout &);
	virtual void visit(BasicTypeDeclaration &);
	virtual void visit(ImageTypeDeclaration &);
	virtual void visit(StructDeclaration &);
	virtual void visit(VariableDeclaration &);
	virtual void visit(InterfaceBlock &);
	virtual void visit(FunctionDeclaration &);
	virtual void visit(Conditional &);
	virtual void visit(Iteration &);
	virtual void visit(Passthrough &);
	virtual void visit(Return &);
	virtual void visit(Jump &);
};

} // namespace SL
} // namespace GL
} // namespace Msp

#endif
