#include "glsl_error.h"
#include "parser.h"

using namespace std;

namespace Msp {
namespace GL {
namespace SL {

invalid_shader_source::invalid_shader_source(const Location &loc, const string &message):
	runtime_error(format("%s:%d: %s", loc.name, loc.line, message))
{ }

syntax_error::syntax_error(const Location &loc, const string &token, const string &message):
	invalid_shader_source(loc, "Syntax error near '%s': %s", token, message)
{ }

parse_error::parse_error(const Location &loc, const string &token, const string &expected):
	invalid_shader_source(loc, "Parse error at '%s': expected %s", token, expected)
{ }


Diagnostic::Diagnostic():
	severity(INFO),
	source(0),
	line(0),
	provoking_source(0),
	provoking_line(0)
{ }

Diagnostic::Diagnostic(Severity v, int s, unsigned l, const string &m):
	severity(v),
	source(s),
	line(l),
	provoking_source(s),
	provoking_line(l),
	message(m)
{ }

} // namespace SL
} // namespace GL
} // namespace Msp
